#!/bin/bash

# Written 2009, 2010  Eric Hameleers, Eindhoven, NL

# Convert 32bit slackware packages 'en masse' into compatibility packages
# to be installed on slackware64 - providing the 32bit part of multilib.
# The string "-compat32" is added at the end of package name when a
# compatibility package gets created.  This allows it to be installed
# on slackware64 alongside the native 64bit versions.
# For example: the original 32bit package "bzip2" will be converted to a new
#              package with the name "bzip2-compat32"

# You also have to install multilib versions of glibc and gcc !

# Before we start
[ -x /bin/id ] && CMD_ID="/bin/id" || CMD_ID="/usr/bin/id"
if [ "$($CMD_ID -u)" != "0" ]; then
  echo "You need to be root to run $(basename $0)."
  exit 1
fi

# Should we be verbose?
VERBOSE=${VERBOSE:-1}

# In case we need temporary storage:
TMP=${TMP:-/tmp}
mkdir -p $TMP
if [ ! -w "$TMP" ]; then
  echo "Can not write to temporary directory '$TMP'!"
  exit 3
fi

# Zero some other variables:
SLACK32ROOT=""
SLACK32URL=""
TARGET64ROOT=""

# Helpful instructions in case the user asks for it:
function show_help () {
  # Write the help text to output:
  cat <<EOF

Usage: $0 <-i 32bit_package_tree|-u 32bit_package_url>  [-d output_directory]

$(basename $0) converts an essential subset of 32-bit Slackware
packages into 'compatibility' packages for 64-bit Slackware.

Required parameter - one of these two::
  -i 32bit_package_tree        A 32bit Slackware package-tree. It should have
                               the a,ap,d,..,y directories immediately below.
  -u 32bit_package_url         The URL of a http or ftp server containing 32bit
                               Slackware packages. It should have the
                               a,ap,d,..,y directories immediately below.
Optional parameter::
  -d destination_directory     create packages in this directory.
                               By default, the new packages will be created
                               in your current directory.

Example of a useable Slackware URL:
  http://slackware.mirrors.tds.net/pub/slackware/slackware-13.0/slackware

EOF
}

# Parse the commandline parameters:
while [ ! -z "$1" ]; do
  case $1 in
    -d|--destdir)
      TARGET64ROOT="$(cd $(dirname "${2}"); pwd)/$(basename "${2}")"
      shift 2
      ;;
    -h|--help)
      show_help
      exit 0
      ;;
    -i|--inputdir)
      SLACK32ROOT="$(cd $(dirname "${2}"); pwd)/$(basename "${2}")"
      shift 2
      ;;
    -u|--inputurl)
      SLACK32URL="${2}"
      shift 2
      ;;
    -*)
      echo "Unsupported parameter '$1'!"
      exit 1
      ;;
    *)
      # Do nothing
      shift
      ;;
  esac
done

# A function to determine if there are spaces in the pathname:
function contains_spaces () {
  local CHOPPED=$(echo "$1" | tr -d ' ')
  [ "x$CHOPPED" = "x$1" ] && return 1 || return 0
}

# A function to download a package:
function download_pkg () {
  local IP="$1"
  local BP="$(basename $1)"
  local DP="$(dirname $1)"

  local PS="$(basename $DP)"

  local DURL="$2"

  mkdir -p "$DP"
  cd "$DP"
  echo "--- Downloading '$PS/$BP'..."
  lftp -c 'open '${DURL}' ; mget '${PS}/${BP}'-*.t?z'
}

# A function to retrieve the fullname of a package:
function get_pkgfullpath () {
  local IP="$1"
  local BP="$(basename $1)"
  local FP=""

  for FP in $(ls ${IP}-*.t?z) ; do
    if [ "$(echo "$FP"|rev|cut -d- -f4-|cut -d/ -f1| rev)" = "$BP" ];
    then
      echo "$FP"
      continue
    fi
  done
}

# Safety checks in case a URL was provided: 
if [ -n "$SLACK32URL" ]; then
  if [ -n "$SLACK32ROOT" ]; then
    echo "*** Options '-i' and '-u' can not be used together!"
    exit 1
  else
    # Define a 'temporary' root directory where we will download packages:
    SLACK32ROOT="${TMP}/alienBOB"
    if ! which lftp 1>/dev/null 2>&1 ; then
      echo  "No lftp binary detected! Need lftp for package downloading!"
      exit 1
    fi
  fi
fi

# The root directory of 32bit Slackware packages
# (should have the a,ap,d,..,y directories immediately below):
# Let's use a fallback directory in case none was specified:
SLACK32ROOT="${SLACK32ROOT:-"/home/ftp/pub/Linux/Slackware/slackware-current/slackware"}"

# The output directory for our converted packages; defaults to the current dir.
# Note that {a,ap,d,l,n,x}-compat32 directories will be created below this
# directory if they do not yet exist:
TARGET64ROOT="${TARGET64ROOT:-"$(pwd)"}"

# Abort if we got directories with spaces in them:
if contains_spaces "$SLACK32ROOT" ; then
  echo "Directories with spaces are unsupported: '$SLACK32ROOT'!"
  exit 1
fi
if contains_spaces "$TARGET64ROOT" ; then
  echo "Directories with spaces are unsupported: '$TARGET64ROOT'!"
  exit 1
fi

# Where the scripts are:
SRCDIR=$(cd $(dirname $0); pwd)

# The script that does the package conversion:
CONV32=$SRCDIR/convertpkg-compat32

# Bail out if the conversion script is not available/working:
if [ ! -f $CONV32 ]; then
  echo "Required script '$CONV32' is not present or not executable! Aborting..."
  exit 1
fi

# We can not proceed if there are no packages and we did not get an URL:
if [ -z "$SLACK32URL" ]; then
  if [ ! -d $SLACK32ROOT/a -o ! -d $SLACK32ROOT/ap -o ! -d $SLACK32ROOT/d -o ! -d $SLACK32ROOT/l -o ! -d $SLACK32ROOT/n -o ! -d $SLACK32ROOT/x ]; then
    echo "Required package directories a,ap,d,l,n,x below '$SLACK32ROOT' are not found! Aborting..."
    exit 1
  fi
fi

# If a destination_directory was specified, abort now if we can not create it:
if [ -n "$TARGET64ROOT" -a ! -d "$TARGET64ROOT" ]; then
  echo "Creating output directory '$TARGET64ROOT'..."
  mkdir -p $TARGET64ROOT
  if [ ! -w "$TARGET64ROOT" ]; then
    echo "Creation of output directory '$TARGET64ROOT' failed!"
    exit 3
  fi
fi

# This is the script's internal list of what I consider as the essential
# 32bit multilib package set for your Slackware64:

# The A/ series:
A_COMPAT32="
bzip2
cups
cxxlibs
dbus
e2fsprogs
openssl-solibs
util-linux-ng
"

# The AP/ series:
AP_COMPAT32="
mpg123
mysql
"

# The D/ series:
D_COMPAT32="
libtool
"

# The L/ series:
L_COMPAT32="
alsa-lib
alsa-oss
atk
audiofile
cairo
dbus-glib
esound
expat
freetype
gamin
glib2
gtk+2
hal
jasper
lcms
libart_lgpl
libexif
libgphoto2
libjpeg
libidn
libmng
libpng
libtermcap
libtiff
libxml2
libxslt
ncurses
pango
popt
qt
readline
sdl
seamonkey-solibs
svgalib
v4l-utils
zlib
"

# The N/ series:
N_COMPAT32="
curl
cyrus-sasl
gnutls
libgcrypt
libgpg-error
openldap-client
openssl
"

# The X/ series:
X_COMPAT32="
fontconfig
glew
libFS
libICE
libSM
libX11
libXScrnSaver
libXTrap
libXau
libXaw
libXcomposite
libXcursor
libXdamage
libXdmcp
libXevie
libXext
libXfixes
libXfont
libXfontcache
libXft
libXi
libXinerama
libXmu
libXp
libXpm
libXprintUtil
libXrandr
libXrender
libXres
libXt
libXtst
libXv
libXvMC
libXxf86dga
libXxf86misc
libXxf86vm
libdmx
libdrm
libfontenc
libxcb
mesa
pixman
"

# Create target directories if they do not exist:
for TDIR in a-compat32 ap-compat32 d-compat32 l-compat32 n-compat32 x-compat32 ; do
  mkdir -p $TARGET64ROOT/$TDIR
  if [ ! -w $TARGET64ROOT/$TDIR ]; then
    echo "Directory '$TARGET64ROOT/$TDIR' is not writable! Aborting..."
    exit 1
  fi
done

# Convert the 32bit packages from A AP D L N and X series:
echo "***"
echo "*** Starting the conversion process:"

echo "*** 'A' series:"
for INPKG in $A_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/a/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/a/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/a-compat32
done

echo "*** 'AP' series:"
for INPKG in $AP_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/ap/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/ap/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/ap-compat32
done

echo "*** 'D' series:"
for INPKG in $D_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/d/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/d/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/d-compat32
done

echo "*** 'L' series:"
for INPKG in $L_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/l/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/l/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/l-compat32
done

echo "*** 'N' series:"
for INPKG in $N_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/n/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/n/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/n-compat32
done

echo "*** 'X' series:"
for INPKG in $X_COMPAT32 ; do
  [ -n "$SLACK32URL" ] && download_pkg $SLACK32ROOT/x/$INPKG $SLACK32URL
  FULLPKG=$(get_pkgfullpath $SLACK32ROOT/x/$INPKG)
  if [ -z "$FULLPKG" ]; then
    echo "*** FAIL: package '$INPKG' was not found!"
    continue
  fi
  [ "$VERBOSE" = "1" ] && echo "--- $INPKG"
  sh $CONV32 -i $FULLPKG -d $TARGET64ROOT/x-compat32
done

# Mention downloaded packages if we used a URL as source:
if [ -n "$SLACK32URL" ]; then
  echo "WARNING: packages which were downloaded from '$SLACK32URL'"
  echo "have been left in directory '$SLACK32ROOT'."
  echo "It is safe to remove these now."
  echo ""
fi

echo "*** Conversion done!"
echo "***"
